package org.artifactory.projects;

import lombok.NonNull;
import org.jfrog.access.client.AccessClientException;
import org.jfrog.access.common.EnvironmentsConstants;

import java.util.List;
import java.util.Set;

/**
 * @author BarakH
 */
public interface ProjectsService {

    String PROJECT = "project";
    String DEFAULT_PROJECT = "default";
    String ENVIRONMENTS = "environments";
    String ENV_DEV = EnvironmentsConstants.DEV;
    String ENV_PROD = EnvironmentsConstants.PROD;
    List<String> GLOBAL_ENVIRONMENTS = List.of(ENV_PROD, ENV_DEV);

    /**
     * Add a new repository to the given project and environment
     */
    default void addProjectRepository(String project, List<String> environments, String repoKey) throws
            AccessClientException {
    }

    /**
     * Remove a repo resource from its current project
     */
    default void removeRepositoryFromCurrentProject(String repoKey) throws AccessClientException {
    }

    /**
     * @return A list with all the projects the user has ProjectAdmin role on.
     */
    @NonNull
    List<String> getAllProjectsManagedByUser();

    /**
     * @return A list of all the project repositories
     */
    @NonNull
    Set<String> getProjectRepositories(String project);

    /**
     * @return A set of all the repositories the user has admin role over
     */
    @NonNull
    Set<String> getProjectAdminRepositories();

    /**
     * @return The repository project name or default project if none
     */
    @NonNull
    String getProjectOfRepository(String repoKey);

    /**
     * @throws IllegalArgumentException if the repo key is not matching to the given project prefix or the
     *                                  environments are not configured for the project.
     */
    default void validateRepoConfigIsMatchingToProjectSpec(String project, String repoKey, List<String> environments)
            throws IllegalArgumentException {
    }
}
