package org.artifactory.metrics.framework;

import com.jfrog.sysconf.SysConfig;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.repo.http.HttpConnectionPoolMonitorService;
import org.artifactory.security.access.AccessService;
import org.artifactory.storage.binstore.service.GarbageCollectionDatastoreService;
import org.artifactory.storage.db.util.JdbcHelper;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.artifactory.test.TestUtils;
import org.junit.Before;
import org.mockito.Mock;
import org.testng.annotations.Test;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

public class MetricsServiceImplTest extends ArtifactoryHomeBoundTest {

    @Mock
    private SysConfig sysConfig;
    @Mock
    private AccessService accessService;
    @Mock
    private GarbageCollectionDatastoreService garbageCollectionDatastoreService;
    @Mock
    private HttpConnectionPoolMonitorService httpConnectionPoolMonitorService;
    @Mock
    private JdbcHelper jdbcHelper;


    @Before
    public void setUp() {
        TestUtils.setField(ArtifactoryHome.get(), "sysConfig", sysConfig);
    }

    @Test
    public void testMetricsService() {
        MetricsServiceImpl metricsService =
                new MetricsServiceImpl(accessService, garbageCollectionDatastoreService, jdbcHelper,
                        httpConnectionPoolMonitorService);
        SysConfig sysConfig = mock(SysConfig.class);
        TestUtils.setField(ArtifactoryHome.get(), "sysConfig", sysConfig);

        when(sysConfig.getBoolean("artifactory.metrics.enabled", false)).thenReturn(false);
        metricsService.onContextCreated();
        assertFalse(metricsService.isMetricServiceRunning());

        when(sysConfig.getBoolean("artifactory.metrics.enabled", false)).thenReturn(true);
        when(sysConfig.get("artifactory.metrics.exclude", "")).thenReturn("");
        metricsService.onContextCreated();
        assertTrue(metricsService.isMetricServiceRunning());
    }

}