package org.artifactory.metrics.framework.providers;

import org.artifactory.storage.db.init.ArtifactoryDataSource;
import org.artifactory.storage.db.util.JdbcHelper;
import org.jfrog.metrics.model.Metric;
import org.testng.annotations.Test;

import java.sql.SQLException;
import java.util.List;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

public class DatabaseConnectionMetricProviderTest {

    @Test
    public void testGetName() {
        DatabaseConnectionMetricProvider dbProvider =
                new DatabaseConnectionMetricProvider(null);
        assertEquals("jfrt_db_connections", dbProvider.getName());
    }

    @Test
    public void testGetDescription() {
        DatabaseConnectionMetricProvider dbProvider =
                new DatabaseConnectionMetricProvider(null);
        assertEquals("Artifactory Database Connection Metrics Provider",
                dbProvider.getDescription());
    }

    @Test
    public void testGetRefreshIntervalMillis() {
        DatabaseConnectionMetricProvider dbProvider =
                new DatabaseConnectionMetricProvider(null);
        assertEquals(60000, dbProvider.getRefreshIntervalMillis());
    }

    @Test
    public void testCollectDBMetric() throws SQLException {
        JdbcHelper jdbcHelper = mock(JdbcHelper.class);

        ArtifactoryDataSource ds = mock(ArtifactoryDataSource.class);
        when(ds.isWrapperFor(ArtifactoryDataSource.class)).thenReturn(true);
        when(ds.unwrap(ArtifactoryDataSource.class)).thenReturn(ds);
        when(jdbcHelper.getDataSource()).thenReturn(ds);
        when(ds.getActiveConnectionsCount()).thenReturn(10);
        when(ds.getIdleConnectionsCount()).thenReturn(10);
        when(ds.getMaxActive()).thenReturn(100);
        when(ds.getMaxIdle()).thenReturn(10);
        when(ds.getMinIdle()).thenReturn(10);
        DatabaseConnectionMetricProvider dbProvider =
                new DatabaseConnectionMetricProvider(jdbcHelper);
        List<Metric> metrics = dbProvider.collect();

        assertEquals(metrics.get(0).getName(), "active");
        assertEquals(metrics.get(0).getValue(), 10.0, 0.1);
        assertEquals(metrics.get(1).getName(), "idle");
        assertEquals(metrics.get(1).getValue(), 10.0, 0.1);
        assertEquals(metrics.get(2).getName(), "max_active");
        assertEquals(metrics.get(2).getValue(), 100.0, 0.1);
        assertEquals(metrics.get(3).getName(), "min_idle");
        assertEquals(metrics.get(3).getValue(), 10.0, 0.1);

    }
}
