package org.artifactory.traffic;

import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

public class TrafficTimeBasedRollingFileAppenderTest {

    private TestChild testChild;

    @BeforeMethod
    public void setUp() {
        testChild = new TestChild();
    }

    @Test
    public void whenStopThenExecutorShutdown() {
        testChild.setIntervalMs(1000L);
        testChild.stop();
        assertTrue(testChild.executorShutdown);
    }

    @Test
    public void whenStopIfStartTrafficRolloverSystemPropertySetToFalseThenExecutorNotShutdown() {
        testChild.setIntervalMs(1000L);
        testChild.startTrafficRolloverThreadSysProp = "false";
        testChild.stop();
        assertFalse(testChild.executorShutdown);
    }

    @Test
    public void whenIntervalNotSetThenNoRolloverScheduled() {
        testChild.start();
        assertFalse(testChild.rolloverScheduled);
    }

    @Test
    public void whenIntervalSetThenRolloverScheduled() {
        testChild.setIntervalMs(1000L);
        testChild.start();
        assertTrue(testChild.rolloverScheduled);
    }

    @Test
    public void whenStartTrafficRolloverSystemPropertySetToFalseThenRolloverNoScheduled() {
        testChild.startTrafficRolloverThreadSysProp = "false";
        testChild.setIntervalMs(1000L);
        testChild.start();
        assertFalse(testChild.rolloverScheduled);
    }

    @Test
    public void whenCalculateInitialDelayThenLifeTimeTakenIntoAccount() {
        testChild.setIntervalMs(1000L);
        testChild.lifetime = 500;
        testChild.start();
        assertEquals(500, testChild.initialDelay);
    }

    private static class TestChild extends TrafficTimeBasedRollingFileAppender {

        private boolean executorShutdown = false;
        private boolean rolloverScheduled = false;
        private long initialDelay = 0;
        private long lifetime = 0;
        private String startTrafficRolloverThreadSysProp = "true";

        @Override
        protected String getStartTrafficRolloverSystemProperty() {
            return startTrafficRolloverThreadSysProp;
        }

        @Override
        protected void shutdownExecutor() {
            executorShutdown = true;
        }

        @Override
        protected void scheduleRollover(long initialDelay) {
            this.initialDelay = initialDelay;
            rolloverScheduled = true;
        }

        @Override
        protected long calcLifetime() {
            return lifetime;
        }

    }
}
