package org.artifactory.concurrent;

import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.opentracing.ArtifactoryJaegerTracerConfiguration;
import org.artifactory.opentracing.ArtifactoryTracer;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

import static org.mockito.Mockito.when;
import static org.testng.Assert.assertNull;
import static org.testng.AssertJUnit.assertNotNull;

/**
 * @author Uriah Levy
 */
@Test
public class ArtifactoryExecutorTest {

    private ArtifactoryTracer tracer = new ArtifactoryTracer(ArtifactoryJaegerTracerConfiguration.initTracer());

    @Mock
    ArtifactoryContext artifactoryContext;


    @BeforeClass
    public void setUp() {
        MockitoAnnotations.initMocks(this);
        when(artifactoryContext.beanForType(ArtifactoryTracer.class))
                .thenReturn(tracer);
    }

    @Test(expectedExceptions = UnsupportedOperationException.class)
    public void testFixedThreadPool() {
        ArtifactoryExecutorFactory.newFixedThreadPool(1, artifactoryContext, true);
    }

    public void testNotBoundClassicThreadPool() throws ExecutionException, InterruptedException {
        ArtifactoryExecutor executor = ArtifactoryExecutorFactory
                .newClassicThreadPool(ThreadPoolConfig.builder()
                        .maxThreads(1)
                        .queue(new LinkedBlockingQueue<>())
                        .keepAliveTimeout(1L)
                        .keepAliveTimeoutTimeUnit(TimeUnit.SECONDS)
                        .build(), artifactoryContext, false);
        // Not bound
        executor.submit(() -> assertNull(ContextHelper.get())).get();
    }

    public void testBoundClassicThreadPool() throws ExecutionException, InterruptedException {
        ArtifactoryExecutor executor = ArtifactoryExecutorFactory
                .newClassicThreadPool(ThreadPoolConfig.builder()
                        .maxThreads(1)
                        .queue(new LinkedBlockingQueue<>())
                        .keepAliveTimeout(1L)
                        .keepAliveTimeoutTimeUnit(TimeUnit.SECONDS)
                        .build(), artifactoryContext, true);
        // Bound
        executor.submit(() -> assertNotNull(ContextHelper.get())).get();
    }

    public void testTraceIdOnExecutorThread() throws ExecutionException, InterruptedException {
        ArtifactoryExecutor executor = ArtifactoryExecutorFactory
                .newClassicThreadPool(ThreadPoolConfig.builder()
                        .maxThreads(1)
                        .queue(new LinkedBlockingQueue<>())
                        .keepAliveTimeout(1L)
                        .keepAliveTimeoutTimeUnit(TimeUnit.SECONDS)
                        .build(), artifactoryContext, false);
        // Trace ID exists
        executor.submit(() -> {
            assertNull(ContextHelper.get());
            assertNotNull(tracer.getFullContextTracingId());
        }).get();
    }
}