:: Helper to work with yaml files

call :initYQ

:: Assign method name and its parameters
set method=%1
set ALL_BUT_FIRST=
set SENSITIVE_KEY_VALUE=__sensitive_key_hidden___

for /f "tokens=1,* delims= " %%a in ("%*") do set ALL_BUT_FIRST=%%b
:: Use /1 to retain %0 values intact
shift /1

goto %method% %ALL_BUT_FIRST%
goto :eof

:: Transform given variable content to jfrog equivalent environment variable and assign it to 2nd argument
:: If no variable is passed TRANSFORMED_KEY will be set with 
:: This will replace,
::      - lower case with upper case alphabets
::      - hyphen with underscore
::      - dot with underscore
::
:: Sample usage :
::    SET String="jf.product.home"
::    CALL :transformKeyToEnv %String% modified
::    echo %modified%      :: output : JF_PRODUCT_HOME
:transformKeyToEnv
    set envKey=%~1
    set prefix=JF_

    if not defined envKey goto :transformKeyToEnvEnd

    :: replace dot and underscore with underscore
    SET envKey=!envKey:.=_!
    SET envKey=!envKey:-=_!

    :: convert all lowercase alphabets to upper case 
    call :toUpperCase envKey

    :: Add prefix if its not already added
    if not "!envKey:~0,3!" == "!prefix!" set envKey=%prefix%%envKey%
 
    :: set the second argument to transformed value
    if not [%~2] == [] (
        set %~2=!envKey!
    ) else (
        set "TRANSFORMED_KEY=!envKey!"
    )

    :transformKeyToEnvEnd
exit /b 0

:: Change given variable content to uppercase
:: Sample usage :
::    SET String="jf.product.home"
::    CALL :toUpperCase String
::    echo %String%      :: output : JF.PRODUCT.HOME
:toUpperCase
     if not defined %~1 goto :toUpperCaseEnd
    for %%i in ("a=A" "b=B" "c=C" "d=D" "e=E" "f=F" "g=G" "h=H" "i=I"
            "j=J" "k=K" "l=L" "m=M" "n=N" "o=O" "p=P" "q=Q" "r=R"
            "s=S" "t=T" "u=U" "v=V" "w=W" "x=X" "y=Y" "z=Z" "ä=Ä"
            "ö=Ö" "ü=Ü") do (
        call set "%1=%%%1:%%~i%%"
    )
    :toUpperCaseEnd
EXIT /b 0

REM # Check if a give key contains any sensitive string as part of it
REM # This will set isSensitive variable with value 'yes' if it contains sensitive string
REM # Based on the result, the caller can decide its value can be displayed or not
REM #   Sample usage : 
REM #        call :isKeySensitive "%key%"
REM #        if "!isSensitive!" == "yes" set displayValue=__sensitive_key_hidden___
:isKeySensitive
    set Key=%~1
    set sensitiveKeys=password,secret,key,token
    set isSensitive=no

    for %%a in ("%sensitiveKeys:,=" "%") do (
        echo.%key% | findstr /C:"%%a" 1>nul 2>&1
        if !ERRORLEVEL! == 0 (
            set isSensitive=yes
            goto :isKeySensitiveEnd
        ) 
    )

    :isKeySensitiveEnd
exit /b 0

:resolutionlogger
    setlocal
    set key=%~1
    set yamlValue="%~2"
    set valueSource=%~3
    
    set isEmpty=no
    if not defined key set isEmpty=yes
    if not defined yamlValue set isEmpty=yes
    if not defined valueSource set isEmpty=yes

    if "%isEmpty%" == "yes" goto :resolutionloggerEnd

    call :isKeySensitive "%key%"
    if "!isSensitive!" == "yes" set yamlValue=!SENSITIVE_KEY_VALUE!

    SET yamlPrint=!yamlValue:^<=@gt@!
    SET yamlPrint=!yamlPrint:^>=@lt@!
    SET yamlPrint=!yamlPrint:^"='!

    CALL %~dp0\artifactoryCommon.bat :log "resolved !key! with value %yamlPrint% from !valueSource!"

    :resolutionloggerEnd
    endlocal
exit /b 0

:initYQ
    if defined YQ_PATH if exist %YQ_PATH%\yq.exe (
        set YQ="%YQ_PATH%\yq.exe"
        goto :initYQCheck
    )

    set defaultYQLocation="%JF_PRODUCT_HOME%\app\third-party\yq\yq.exe"

    if defined JF_PRODUCT_HOME if exist "%defaultYQLocation%" (
        set YQ=%defaultYQLocation%
        goto :initYQCheck
    )

    if not defined YQ (
        CALL %~dp0\artifactoryCommon.bat :warn "Unable to find yq, please set YQ_PATH to parent directory of yq binary"
        goto :initYQEnd
    )

    :initYQCheck
    "%YQ%" -V >nul 2>&1
    :: check if yq is working
    if not !ERRORLEVEL! == 0 CALL %~dp0\artifactoryCommon.bat :warn "Unable to get version from yq executable, it might be corrupt or incompatible. Command used - %YQ% -V"

    :initYQEnd
exit /b 0

REM #######################################
REM # Get value from a yaml file
REM # Globals:
REM #   YAML_VALUE
REM #   YQ
REM # Arguments:
REM #   key           - key to be searched
REM #   yamlFile      - target yaml file
REM #   logResolution - logs a message about the file which had this key set
REM # Returns:
REM #   None
REM #######################################
:getYamlValue
    set yamlKey=%~1
    set yamlFile=%~2
    set logResolution=%~3

    set yamlValue=
    set YAML_VALUE=

    if not defined yamlKey goto :getYamlValueEnd
    if not defined yamlFile set yamlFile=%JF_PRODUCT_HOME%\var\etc\system.yaml
    if not defined logResolution set logResolution=true

    if not exist !yamlFile! goto :getYamlValueEnd

    for /f "tokens=*" %%f in ('%YQ% r !yamlFile! %yamlKey%') do (
        if not %%f == null (
            if defined yamlValue ( set "yamlValue=!yamlValue! %%f" ) else ( set "yamlValue=%%f" )
        )
    )

    if not defined yamlValue goto :getYamlValueEnd

    if defined yamlValue if "!yamlValue!" == "null" (
        set yamlValue=
        goto :getYamlValueEnd
    )

    if defined yamlValue if "!logResolution!" == "true" (
        call :resolutionlogger "!yamlKey!" "!yamlValue!" "!yamlFile!"
    )

    set "YAML_VALUE=!yamlValue!"
    :getYamlValueEnd
exit /b 0

REM #######################################
REM # Get equivalent environment variable of yaml key
REM #   If a key is parent.child, the equivalent environment variable is PARENT_CHILD
REM # Globals:
REM #   YAML_VALUE
REM # Arguments:
REM #   key           - key to be searched
REM #   logResolution - logs a message if key was set as an environment variable
REM # Returns:
REM #   None
REM #######################################
:getENVValue
    set key=%~1
    set logResolution=%~2
    set YAML_VALUE=
    set envValue=

    if not defined logResolution set logResolution=true

    if not defined key goto :getENVValueEnd

    call :transformKeyToEnv "!key!" envValue

    if not defined envValue goto :getENVValueEnd
    if [!%envValue%!] == [] goto :getENVValueEnd

    if "!logResolution!" == "true" (
        call :resolutionlogger "!key!" "!%envValue%!" "environment variable"
    )

    set "YAML_VALUE=!%envValue%!"

    :getENVValueEnd
exit /b 0

REM #######################################
REM # Get value from a system yaml file
REM #   Set JF_SYSTEM_YAML with the list of files to be parsed through
REM # Globals:
REM #   YAML_VALUE
REM #   YQ
REM # Arguments:
REM #   key           - key to be searched
REM #   default       - a default value to set if none of the files have this key
REM #   logResolution - logs a message about the file which had this key set
REM # Returns:
REM #   None
REM #######################################
:getSystemValue
    set key=%~1
    set default=%~2
    set logResolution=%~3
    set yamlFile=

    if not defined key goto :getSystemValueEnd
    if not defined logResolution set logResolution=true
    set yamlFile=%JF_SYSTEM_YAML%
    if defined JF_SYSTEM_YAML ( set "yamlFile=%JF_SYSTEM_YAML%" ) else ( set "yamlFile=%JF_PRODUCT_HOME%\var\etc\system.yaml" )
    set YAML_VALUE=

    :: check if this key is set as an environment variable
    call :getENVValue "!key!" "!logResolution!"
    if defined YAML_VALUE goto :getSystemValueEnd

    :: check if the key is set in any of the yaml files
    call :getYamlValue "!key!" "!yamlFile!" "!logResolution!"

    if defined YAML_VALUE goto :getSystemValueEnd
    if defined default set YAML_VALUE=%default%

    :getSystemValueEnd
exit /b 0 

REM #######################################
REM # escapeSpecialCharacters
REM # Replace @gt@ with ^^^< and @lt@ with ^^^>
REM #######################################
:escapeSpecialCharacters
    set value=%1    
    set gt=^^^<
    set lt=^^^>

	set value=!value:@gt@=%gt%!
	set value=!value:@lt@=%lt%!
    
exit /B 0

REM #######################################
REM # Get value from a system yaml file
REM #   Optionally set JF_SYSTEM_YAML with the target system.yaml to be updated
REM # Globals:
REM #   YAML_VALUE
REM #   YQ
REM # Arguments:
REM #   key           - key to be modified
REM #   value         - value to be set
REM # Returns:
REM #   None
REM #######################################
:setSystemValue
    setlocal EnableDelayedExpansion
    set key=%~1
    set value=%2
    set logResolution=%~3
    set yamlFile=
    set displayValue=
    set isEmpty=no
	
    if not defined key goto :setSystemValueEnd
    if not defined value goto :setSystemValueEnd
    if not defined logResolution set logResolution=true
    if defined JF_SYSTEM_YAML ( set yamlFile="%JF_SYSTEM_YAML%" ) else ( set yamlFile="%JF_PRODUCT_HOME%\var\etc\system.yaml" )

    call :isKeySensitive "%key%"
    if "!isSensitive!" == "yes" ( set displayValue=!SENSITIVE_KEY_VALUE! ) else ( set displayValue=%value% )
    
    :: Check if file is not available
    if not exist "%yamlFile%" (
        set "isEmpty=yes" 
        goto :yqExec 
    ) 

    :: Check if file is empty
    set size=0
    for /f %%i in ("%yamlFile%") do set size=%%~zi
    if %size% lss 1 set isEmpty=yes
	
    :yqExec
    call :escapeSpecialCharacters !value!
    if "%isEmpty%" == "yes" (
        "%YQ%" n "%key%" %value% >"%yamlFile%"
    ) else (
        "%YQ%" w -i "%yamlFile%" "%key%" %value%
    )
    :setSystemValueEnd
    endlocal
exit /b 0